@@ -32,8 +32,8 @@
  * X-macro generic impl, included into the various LimEx model implementations.
  */
 
-#if !defined(SIZE) || !defined(STATE_T)
-#  error Must define SIZE and STATE_T in includer.
+#if !defined(SIZE) || !defined(STATE_T) || !defined(LOAD_FROM_ENG)
+#  error Must define SIZE, STATE_T, LOAD_FROM_ENG in includer.
 #endif
 
 #include "config.h"
@@ -44,8 +44,6 @@
 #define PE_FN                   JOIN(processExceptional, SIZE)
 #define RUN_EXCEPTION_FN        JOIN(runException, SIZE)
 #define ZERO_STATE              JOIN(zero_, STATE_T)
-#define LOAD_STATE              JOIN(load_, STATE_T)
-#define STORE_STATE             JOIN(store_, STATE_T)
 #define AND_STATE               JOIN(and_, STATE_T)
 #define EQ_STATE(a, b)          (!JOIN(noteq_, STATE_T)((a), (b)))
 #define OR_STATE                JOIN(or_, STATE_T)
@@ -59,7 +57,7 @@
 #define ESTATE_ARG STATE_T estate
 #else
 #define ESTATE_ARG const STATE_T *estatep
-#define estate LOAD_STATE(estatep)
+#define estate (*estatep)
 #endif
 
 #ifdef STATE_ON_STACK
@@ -97,7 +95,6 @@ int RUN_EXCEPTION_FN(const EXCEPTION_T *e, STATE_ARG,
                      STATE_T *local_succ,
 #endif
                      const struct IMPL_NFA_T *limex,
-                     const ReportID *exReports,
                      u64a offset,
                      struct CONTEXT_T *ctx,
                      struct proto_cache *new_cache,
@@ -133,7 +130,7 @@ int RUN_EXCEPTION_FN(const EXCEPTION_T *e, STATE_ARG,
         char *repeat_state = ctx->repeat_state + info->stateOffset;
 
         if (e->trigger == LIMEX_TRIGGER_POS) {
-            char cyclic_on = TESTBIT_STATE(STATE_ARG_P, info->cyclicState);
+            char cyclic_on = TESTBIT_STATE(*STATE_ARG_P, info->cyclicState);
             processPosTrigger(repeat, repeat_ctrl, repeat_state, offset,
                               cyclic_on);
             *cacheable = DO_NOT_CACHE_RESULT_AND_FLUSH_BR_ENTRIES;
@@ -149,8 +146,7 @@ int RUN_EXCEPTION_FN(const EXCEPTION_T *e, STATE_ARG,
                 *cacheable = DO_NOT_CACHE_RESULT_AND_FLUSH_BR_ENTRIES;
                 DEBUG_PRINTF("stale history, squashing cyclic state\n");
                 assert(e->hasSquash == LIMEX_SQUASH_TUG);
-                STORE_STATE(succ, AND_STATE(LOAD_STATE(succ),
-                            LOAD_STATE(&e->squash)));
+                *succ = AND_STATE(*succ, LOAD_FROM_ENG(&e->squash));
                 return 1; // continue
             } else if (rv == TRIGGER_SUCCESS_CACHE) {
                 new_cache->br = 1;
@@ -164,7 +160,8 @@ int RUN_EXCEPTION_FN(const EXCEPTION_T *e, STATE_ARG,
     // Some exceptions fire accepts.
     if (e->reports != MO_INVALID_IDX) {
         if (flags & CALLBACK_OUTPUT) {
-            const ReportID *reports = exReports + e->reports;
+            const ReportID *reports =
+                (const ReportID *)((const char *)limex + e->reports);
             if (unlikely(limexRunReports(reports, ctx->callback,
                             ctx->context, offset)
                         == MO_HALT_MATCHING)) {
@@ -188,18 +185,16 @@ int RUN_EXCEPTION_FN(const EXCEPTION_T *e, STATE_ARG,
     // Most exceptions have a set of successors to switch on. `local_succ' is
     // ORed into `succ' at the end of the caller's loop.
 #ifndef BIG_MODEL
-    *local_succ = OR_STATE(*local_succ, LOAD_STATE(&e->successors));
+    *local_succ = OR_STATE(*local_succ, LOAD_FROM_ENG(&e->successors));
 #else
-    STORE_STATE(&ctx->local_succ, OR_STATE(LOAD_STATE(&ctx->local_succ),
-                LOAD_STATE(&e->successors)));
+    ctx->local_succ = OR_STATE(ctx->local_succ, LOAD_FROM_ENG(&e->successors));
 #endif
 
     // Some exceptions squash states behind them. Note that we squash states in
     // 'succ', not local_succ.
-    if (e->hasSquash == LIMEX_SQUASH_CYCLIC ||
-                e->hasSquash == LIMEX_SQUASH_REPORT) {
-        STORE_STATE(succ, AND_STATE(LOAD_STATE(succ),
-                    LOAD_STATE(&e->squash)));
+    if (e->hasSquash == LIMEX_SQUASH_CYCLIC
+        || e->hasSquash == LIMEX_SQUASH_REPORT) {
+        *succ = AND_STATE(*succ, LOAD_FROM_ENG(&e->squash));
         if (*cacheable == CACHE_RESULT) {
             *cacheable = DO_NOT_CACHE_RESULT;
         }
@@ -215,13 +210,12 @@ int RUN_EXCEPTION_FN(const EXCEPTION_T *e, STATE_ARG,
 static really_inline
 int PE_FN(STATE_ARG, ESTATE_ARG, u32 diffmask, STATE_T *succ,
           const struct IMPL_NFA_T *limex, const EXCEPTION_T *exceptions,
-          const ReportID *exReports, u64a offset, struct CONTEXT_T *ctx,
-          char in_rev, char flags) {
+          u64a offset, struct CONTEXT_T *ctx, char in_rev, char flags) {
     assert(diffmask > 0); // guaranteed by caller macro
 
-    if (EQ_STATE(estate, LOAD_STATE(&ctx->cached_estate))) {
+    if (EQ_STATE(estate, ctx->cached_estate)) {
         DEBUG_PRINTF("using cached succ from previous state\n");
-        STORE_STATE(succ, OR_STATE(LOAD_STATE(succ), LOAD_STATE(&ctx->cached_esucc)));
+        *succ = OR_STATE(*succ, ctx->cached_esucc);
         if (ctx->cached_reports && (flags & CALLBACK_OUTPUT)) {
             DEBUG_PRINTF("firing cached reports from previous state\n");
             if (unlikely(limexRunReports(ctx->cached_reports, ctx->callback,
@@ -236,7 +230,7 @@ int PE_FN(STATE_ARG, ESTATE_ARG, u32 diffmask, STATE_T *succ,
 #ifndef BIG_MODEL
     STATE_T local_succ = ZERO_STATE;
 #else
-    STORE_STATE(&ctx->local_succ, ZERO_STATE);
+    ctx->local_succ = ZERO_STATE;
 #endif
 
     // A copy of the estate as an array of GPR-sized chunks.
@@ -254,7 +248,7 @@ int PE_FN(STATE_ARG, ESTATE_ARG, u32 diffmask, STATE_T *succ,
 
     u32 base_index[sizeof(STATE_T) / sizeof(CHUNK_T)];
     base_index[0] = 0;
-    for (u32 i = 0; i < ARRAY_LENGTH(base_index) - 1; i++) {
+    for (s32 i = 0; i < (s32)ARRAY_LENGTH(base_index) - 1; i++) {
         base_index[i + 1] = base_index[i] + POPCOUNT_FN(emask_chunks[i]);
     }
 
@@ -276,31 +270,31 @@ int PE_FN(STATE_ARG, ESTATE_ARG, u32 diffmask, STATE_T *succ,
 #ifndef BIG_MODEL
                                   &local_succ,
 #endif
-                                  limex, exReports, offset, ctx, &new_cache,
-                                  &cacheable, in_rev, flags)) {
+                                  limex, offset, ctx, &new_cache, &cacheable,
+                                  in_rev, flags)) {
                 return PE_RV_HALT;
             }
         } while (word);
     } while (diffmask);
 
 #ifndef BIG_MODEL
-    STORE_STATE(succ, OR_STATE(LOAD_STATE(succ), local_succ));
+    *succ = OR_STATE(*succ, local_succ);
 #else
-    STORE_STATE(succ, OR_STATE(LOAD_STATE(succ), ctx->local_succ));
+    *succ = OR_STATE(*succ, ctx->local_succ);
 #endif
 
     if (cacheable == CACHE_RESULT) {
-        STORE_STATE(&ctx->cached_estate, estate);
+        ctx->cached_estate = estate;
 #ifndef BIG_MODEL
         ctx->cached_esucc = local_succ;
 #else
-        STORE_STATE(&ctx->cached_esucc, LOAD_STATE(&ctx->local_succ));
+        ctx->cached_esucc = ctx->local_succ;
 #endif
         ctx->cached_reports = new_cache.reports;
         ctx->cached_br = new_cache.br;
     } else if (cacheable == DO_NOT_CACHE_RESULT_AND_FLUSH_BR_ENTRIES) {
         if (ctx->cached_br) {
-            STORE_STATE(&ctx->cached_estate, ZERO_STATE);
+            ctx->cached_estate = ZERO_STATE;
         }
     }
 
@@ -314,8 +308,6 @@ int PE_FN(STATE_ARG, ESTATE_ARG, u32 diffmask, STATE_T *succ,
 #undef EQ_STATE
 #undef OR_STATE
 #undef TESTBIT_STATE
-#undef LOAD_STATE
-#undef STORE_STATE
 #undef PE_FN
 #undef RUN_EXCEPTION_FN
 #undef CONTEXT_T
@@ -333,11 +325,9 @@ int PE_FN(STATE_ARG, ESTATE_ARG, u32 diffmask, STATE_T *succ,
 #undef STATE_ARG_NAME
 #undef STATE_ARG_P
 
+#undef IMPL_NFA_T
+
 #undef CHUNK_T
 #undef FIND_AND_CLEAR_FN
-#undef IMPL_NFA_T
-#undef GET_NFA_REPEAT_INFO_FN
-
-// Parameters.
-#undef SIZE
-#undef STATE_T
+#undef POPCOUNT_FN
+#undef RANK_IN_MASK_FN
