@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2017, Intel Corporation
+ * Copyright (c) 2016-2018, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -31,6 +31,10 @@
 #include "common.h"
 #include "data_corpus.h"
 #include "engine_hyperscan.h"
+#if defined(HS_HYBRID)
+#include "engine_chimera.h"
+#include "engine_pcre.h"
+#endif
 #include "expressions.h"
 #include "sqldb.h"
 #include "thread_barrier.h"
@@ -54,7 +58,11 @@
 #include <set>
 #include <thread>
 
+#ifndef _WIN32
 #include <getopt.h>
+#else
+#include "win_getopt.h"
+#endif
 #ifndef _WIN32
 #include <pthread.h>
 #if defined(HAVE_PTHREAD_NP_H)
@@ -87,6 +95,8 @@ namespace /* anonymous */ {
 
 bool display_per_scan = false;
 ScanMode scan_mode = ScanMode::STREAMING;
+bool useHybrid = false;
+bool usePcre = false;
 unsigned repeats = 20;
 string exprPath("");
 string corpusFile("");
@@ -102,7 +112,7 @@ typedef void (*thread_func_t)(void *context);
 
 class ThreadContext : boost::noncopyable {
 public:
-    ThreadContext(unsigned num_in, const EngineHyperscan &db_in,
+    ThreadContext(unsigned num_in, const Engine &db_in,
                   thread_barrier &tb_in, thread_func_t function_in,
                   vector<DataBlock> corpus_data_in)
         : num(num_in), results(repeats), engine(db_in),
@@ -132,6 +142,16 @@ public:
 
     // Apply processor affinity (if available) to this thread.
     bool affine(UNUSED int cpu) {
+
+#if defined(_WIN32)
+        SYSTEM_INFO system_info;
+        GetSystemInfo(&system_info);
+        assert(cpu >= 0 && (DWORD)cpu < system_info.dwNumberOfProcessors);
+        DWORD_PTR mask = 1 << cpu;
+        DWORD_PTR rv = SetThreadAffinityMask(thr.native_handle(), mask);
+        return rv != 0;
+#endif
+
 #ifdef HAVE_DECL_PTHREAD_SETAFFINITY_NP
 #if defined(__FreeBSD__)
         cpuset_t cpuset;
@@ -155,7 +175,7 @@ public:
     unsigned num;
     Timer timer;
     vector<ResultEntry> results;
-    const EngineHyperscan &engine;
+    const Engine &engine;
     unique_ptr<EngineContext> enginectx;
     vector<DataBlock> corpus_data;
 
@@ -181,7 +201,11 @@ void usage(const char *error) {
            " (default: streaming).\n");
     printf("  -V              Benchmark in vectored mode"
            " (default: streaming).\n");
-#ifdef HAVE_DECL_PTHREAD_SETAFFINITY_NP
+#if defined(HS_HYBRID)
+    printf("  -H              Benchmark using Chimera (if supported).\n");
+    printf("  -P              Benchmark using PCRE (if supported).\n");
+#endif
+#if defined(HAVE_DECL_PTHREAD_SETAFFINITY_NP) || defined(_WIN32)
     printf("  -T CPU,CPU,...  Benchmark with threads on these CPUs.\n");
 #endif
     printf("  -i DIR          Don't compile, load from files in DIR"
@@ -214,8 +238,8 @@ struct BenchmarkSigs {
 static
 void processArgs(int argc, char *argv[], vector<BenchmarkSigs> &sigSets,
                  UNUSED unique_ptr<Grey> &grey) {
-    const char options[] = "-b:c:Cd:e:E:G:hi:n:No:p:sS:Vw:z:"
-#ifdef HAVE_DECL_PTHREAD_SETAFFINITY_NP
+    const char options[] = "-b:c:Cd:e:E:G:hHi:n:No:p:PsS:Vw:z:"
+#if defined(HAVE_DECL_PTHREAD_SETAFFINITY_NP) || defined(_WIN32)
         "T:" // add the thread flag
 #endif
         ;
@@ -287,6 +311,14 @@ void processArgs(int argc, char *argv[], vector<BenchmarkSigs> &sigSets,
             usage(nullptr);
             exit(0);
             break;
+        case 'H':
+#if defined(HS_HYBRID)
+            useHybrid = true;
+#else
+            usage("Hybrid matcher not enabled in this build");
+            exit(1);
+#endif
+            break;
         case 'n':
             if (!fromString(optarg, repeats) || repeats == 0) {
                 usage("Couldn't parse argument to -n flag, should be"
@@ -294,6 +326,14 @@ void processArgs(int argc, char *argv[], vector<BenchmarkSigs> &sigSets,
                 exit(1);
             }
             break;
+        case 'P':
+#if defined(HS_HYBRID)
+            usePcre = true;
+#else
+            usage("PCRE matcher not enabled in this build");
+            exit(1);
+#endif
+            break;
         case 's':
             in_sigfile = 2;
             break;
@@ -306,7 +346,7 @@ void processArgs(int argc, char *argv[], vector<BenchmarkSigs> &sigSets,
         case 'S':
             sigName.assign(optarg);
             break;
-#ifdef HAVE_DECL_PTHREAD_SETAFFINITY_NP
+#if defined(HAVE_DECL_PTHREAD_SETAFFINITY_NP) || defined(_WIN32)
         case 'T':
             if (!strToList(optarg, threadCores)) {
                 usage("Couldn't parse argument to -T flag, should be"
@@ -399,6 +439,24 @@ void processArgs(int argc, char *argv[], vector<BenchmarkSigs> &sigSets,
         exit(1);
     }
 
+    // Constraints on Chimera and PCRE engines
+    if (useHybrid || usePcre) {
+        if (useHybrid && usePcre) {
+            usage("Can't run both Chimera and PCRE.");
+            exit(1);
+        }
+        if (scan_mode != ScanMode::BLOCK) {
+            usage("Must specify block mode in Chimera or PCRE with "
+                  "the -N option.");
+            exit(1);
+        }
+
+        if (forceEditDistance || loadDatabases || saveDatabases) {
+            usage("No extended options are supported in Chimera or PCRE.");
+            exit(1);
+        }
+    }
+
     // Read in any -s signature sets.
     for (const auto &file : sigFiles) {
         SignatureSet sigs;
@@ -503,7 +561,7 @@ static
 void benchStreamingInternal(ThreadContext *ctx, vector<StreamInfo> &streams,
                             bool do_compress) {
     assert(ctx);
-    const EngineHyperscan &e = ctx->engine;
+    const Engine &e = ctx->engine;
     const vector<DataBlock> &blocks = ctx->corpus_data;
     vector<char> compress_buf(do_compress ? 1000 : 0);
 
@@ -660,7 +718,11 @@ void displayPerScanResults(const vector<unique_ptr<ThreadContext>> &threads,
         for (size_t j = 0; j != results.size(); j++) {
             const auto &r = results[j];
             double mbps = calc_mbps(r.seconds, bytesPerRun);
+#ifndef _WIN32
             printf("T %2u Scan %2zu: %'0.2f Mbit/sec\n", t->num, j, mbps);
+#else
+            printf("T %2u Scan %2zu: %0.2f Mbit/sec\n", t->num, j, mbps);
+#endif
         }
     }
     printf("\n");
@@ -705,6 +767,7 @@ void displayResults(const vector<unique_ptr<ThreadContext>> &threads,
         }
     }
 
+#ifndef _WIN32
     printf("Time spent scanning:       %'0.3f seconds\n", totalSecs);
     printf("Corpus size:               %'llu bytes ", bytesPerRun);
     switch (scan_mode) {
@@ -720,22 +783,56 @@ void displayResults(const vector<unique_ptr<ThreadContext>> &threads,
         printf("(%'zu blocks)\n", corpus_blocks.size());
         break;
     }
+#else
+    printf("Time spent scanning:       %0.3f seconds\n", totalSecs);
+    printf("Corpus size:               %llu bytes ", bytesPerRun);
+    switch (scan_mode) {
+    case ScanMode::STREAMING:
+        printf("(%zu blocks in %llu streams)\n", corpus_blocks.size(),
+               count_streams(corpus_blocks));
+        break;
+    case ScanMode::VECTORED:
+        printf("(%zu blocks in %llu vectors)\n", corpus_blocks.size(),
+               count_streams(corpus_blocks));
+        break;
+    case ScanMode::BLOCK:
+        printf("(%zu blocks)\n", corpus_blocks.size());
+        break;
+    }
+#endif
 
     u64a totalBytes = bytesPerRun * repeats * threads.size();
     u64a totalBlocks = corpus_blocks.size() * repeats * threads.size();
 
     double matchRate = ((double)matchesPerRun * 1024) / bytesPerRun;
+#ifndef _WIN32
     printf("Matches per iteration:     %'llu (%'0.3f matches/kilobyte)\n",
            matchesPerRun, matchRate);
+#else
+    printf("Matches per iteration:     %llu (%0.3f matches/kilobyte)\n",
+           matchesPerRun, matchRate);
+#endif
 
     double blockRate = (double)totalBlocks / (double)totalSecs;
+#ifndef _WIN32
     printf("Overall block rate:        %'0.2f blocks/sec\n", blockRate);
     printf("Mean throughput (overall): %'0.2Lf Mbit/sec\n",
            calc_mbps(totalSecs, totalBytes));
 
+#else
+    printf("Overall block rate:        %0.2f blocks/sec\n", blockRate);
+    printf("Mean throughput (overall): %0.2Lf Mbit/sec\n",
+           calc_mbps(totalSecs, totalBytes));
+
+#endif
     double lowestScanTime = fastestResult(threads);
+#ifndef _WIN32
     printf("Max throughput (per core): %'0.2Lf Mbit/sec\n",
            calc_mbps(lowestScanTime, bytesPerRun));
+#else
+    printf("Max throughput (per core): %0.2Lf Mbit/sec\n",
+           calc_mbps(lowestScanTime, bytesPerRun));
+#endif
     printf("\n");
 
     if (display_per_scan) {
@@ -812,7 +909,7 @@ void sqlResults(const vector<unique_ptr<ThreadContext>> &threads,
  * the same copy of the data.
  */
 static
-unique_ptr<ThreadContext> makeThreadContext(const EngineHyperscan &db,
+unique_ptr<ThreadContext> makeThreadContext(const Engine &db,
                                             const vector<DataBlock> &blocks,
                                             unsigned id,
                                             thread_barrier &sync_barrier) {
@@ -839,7 +936,7 @@ unique_ptr<ThreadContext> makeThreadContext(const EngineHyperscan &db,
 
 /** Run the given benchmark. */
 static
-void runBenchmark(const EngineHyperscan &db,
+void runBenchmark(const Engine &db,
                   const vector<DataBlock> &corpus_blocks) {
     size_t numThreads;
     bool useAffinity = false;
@@ -848,7 +945,7 @@ void runBenchmark(const EngineHyperscan &db,
         numThreads = 1;
     } else {
         numThreads = threadCores.size();
-#ifdef HAVE_DECL_PTHREAD_SETAFFINITY_NP
+#if defined(HAVE_DECL_PTHREAD_SETAFFINITY_NP) || defined(_WIN32)
         useAffinity = true;
 #else
         useAffinity = false;
@@ -888,7 +985,7 @@ void runBenchmark(const EngineHyperscan &db,
 } // namespace
 
 /** Main driver. */
-int main(int argc, char *argv[]) {
+int HS_CDECL main(int argc, char *argv[]) {
     unique_ptr<Grey> grey;
 #if !defined(RELEASE_BUILD)
     grey = make_unique<Grey>();
@@ -936,8 +1033,18 @@ int main(int argc, char *argv[]) {
                 continue;
             }
 
-            auto engine = buildEngineHyperscan(exprMap, scan_mode, s.name,
-                                               sigName, *grey);
+            unique_ptr<Engine> engine;
+            if (useHybrid) {
+#if defined(HS_HYBRID)
+                engine = buildEngineChimera(exprMap, s.name, sigName);
+            } else if (usePcre) {
+                engine = buildEnginePcre(exprMap, s.name, sigName);
+#endif
+            } else {
+                engine = buildEngineHyperscan(exprMap, scan_mode, s.name,
+                                              sigName, *grey);
+            }
+
             if (!engine) {
                 printf("Error: expressions failed to compile.\n");
                 exit(1);
